@@ -1,23 +1,50 @@
+use std::fmt::Display;
+
 use crate::Indices;
 use crate::{MinMax,here};
 
+/// Maximum value T of slice &[T]
+pub fn maxt<T>(v:&[T]) -> T where T:PartialOrd+Copy {
+    let mut max = &v[0];
+    v.iter().skip(1).for_each(|s| if s > max { max = s }); 
+    *max
+}
+
+/// Minimum value T of slice &[T]
+pub fn mint<T>(v:&[T]) -> T where T:PartialOrd+Copy {
+    let mut min = &v[0];
+    v.iter().skip(1).for_each(|s| if s < min { min = s }); 
+    *min
+}
+
+/// Minimum and maximum (T,T) of a slice &[T]
+pub fn minmaxt<T>(v:&[T]) -> (T,T) where T:PartialOrd+Copy {
+    let mut x1 = &v[0];
+    let mut x2 = x1;
+    v.iter().skip(1).for_each(|s| {
+        if s < x1 { x1 = s } 
+        else if s > x2 { x2 = s }; 
+    });
+    (*x1,*x2)
+}
+
+/// Minimum, minimum's first index, maximum, maximum's first index 
+pub fn minmax<T>(v:&[T])  -> MinMax<T> where T: PartialOrd+Copy {  
+    let (mut min, mut max) = (&v[0],&v[0]); // initialise both to the first item 
+    let (mut minindex,mut maxindex) = (0,0); // indices of min, max
+    v.iter().enumerate().skip(1).for_each(|(i,x)| { 
+        if x < min { min = x; minindex = i } 
+        else if x > max { max = x; maxindex = i }
+    });
+    MinMax{min: *min, minindex, max: *max, maxindex}
+}
+
 /// Reverse a generic slice by reverse iteration.
 /// Creates a new Vec. Its naive use for descending sort etc. 
 /// is to be avoided for efficiency reasons. 
 pub fn revs<T>(s: &[T]) -> Vec<T> where T: Copy 
     { s.iter().rev().copied().collect::<Vec<T>>() }
 
-/// Finds minimum, minimum's first index, maximum, maximum's first index 
-pub fn minmax<T>(v:&[T])  -> MinMax<T> where T: PartialOrd+Copy {  
-    let (mut min, mut max) = (v[0],v[0]); // initialise both to the first item 
-    let (mut minindex,mut maxindex) = (0,0); // indices of min, max
-    v.iter().enumerate().skip(1).for_each(|(i,&x)| { 
-        if x < min { min = x; minindex = i } 
-        else if x > max { max = x; maxindex = i }
-    });
-    MinMax{min,minindex,max,maxindex}
-}
-
 /// Removes repetitions from an explicitly ordered set.
 pub fn sansrepeat<T>(s:&[T]) -> Vec<T> where T: PartialOrd+Copy {
     if s.len() < 2 { return s.to_vec() };
@@ -133,7 +160,7 @@ pub fn memsearch_indexed<T>(s:&[T], i:&[usize], val: T)  -> Option<usize> where
 /// neither smaller nor greater than val. 
 /// When none are found, returns `None`.
 /// Example use: membership of an indexed descending set. 
-pub fn memsearchdesc_indexed<T>(s:&[T], i:&[usize], val: T)  -> Option<usize> where T: PartialOrd {     
+pub fn memsearchdesc_indexed<T>(s:&[T], i:&[usize], val:T)  -> Option<usize> where T: PartialOrd {     
     let n = s.len();
     if n == 0 { return None } // the slice s is empty
     if n == 1 {  // the slice contains a single item 
@@ -211,6 +238,31 @@ pub fn binsearchdesc<T>(s:&[T], val:T) -> usize where T: PartialOrd {
     }  
 }
 
+/// Counts occurrences of val, using previously obtained 
+/// ascending explicit sort `sasc` and descending sort `sdesc`.
+/// This is to facilitate counting of many
+/// different values without ever having to repeat the sorting.
+/// This function is very efficient at counting 
+/// numerous repetitions in large sets (e.g. probabilities in stats).
+/// Binary search from both ends is deployed: O(2log(n)).
+/// # Example:
+/// ```
+/// use crate::indxvec::Indices;
+/// use indxvec::merge::{sortidx,occurs};
+/// let s = [3.141,3.14159,3.14159,3.142];
+/// let sindx = sortidx(&s); // only one sort ever
+/// let sasc = sindx.unindex(&s,true);
+/// let sdesc = sindx.unindex(&s,false);
+/// assert_eq!(occurs(&sasc,&sdesc,3.14159),2);
+/// ```
+pub fn occurs<T>(sasc:&[T],sdesc:&[T],val:T) -> usize where T: PartialOrd+Copy+Display {
+    let ascindex = binsearch(sasc, val); 
+    if ascindex == 0 { return 0 }; // val not found 
+    let descindex = binsearchdesc(sdesc, val);
+    if descindex == 0 { return 0 };
+    ascindex + descindex - sasc.len()
+}
+
 /// Unites two ascending explicitly sorted generic vectors,
 /// by classical selection and copying of their head items into the result.
 /// This is the union of two ordered sets.
